/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.cnd.debugger.common2.values;

import org.openide.util.Utilities;

public class ProcessEvent extends Enum {
    private ProcessEvent(String name) {
	super(name, Catalog.get(name));
    }

    public static final ProcessEvent EXIT =
	new ProcessEvent("Process_exit");		// NOI18N
    public static final ProcessEvent NEXT =
	new ProcessEvent("Process_next");		// NOI18N
    public static final ProcessEvent STEP =
	new ProcessEvent("Process_step");		// NOI18N
    public static final ProcessEvent STOP =
	new ProcessEvent("Process_stop");		// NOI18N
    public static final ProcessEvent LASTRITES =
	new ProcessEvent("Process_lastrites");		// NOI18N
    public static final ProcessEvent GONE =
	new ProcessEvent("Process_gone");	// NOI18N
    public static final ProcessEvent LWP_EXIT =
	new ProcessEvent("Process_lwp_exit");	// NOI18N
    public static final ProcessEvent SYNC =
	new ProcessEvent("Process_sync");	// NOI18N
    public static final ProcessEvent SYNCRTLD =
	new ProcessEvent("Process_syncrtld");	// NOI18N

    private static final ProcessEvent[] linux_enumeration = {
	EXIT, NEXT, STEP, STOP, GONE, SYNCRTLD
    };

    private static final ProcessEvent[] enumeration = {
	EXIT, NEXT, STEP, STOP, LASTRITES, GONE, LWP_EXIT, SYNC, SYNCRTLD
    };

    private static String[] tags;


    public static String[] getTags() {
	if (Utilities.getOperatingSystem() == Utilities.OS_LINUX)
	    tags = makeTagsFrom(tags, linux_enumeration);
	else
	    tags = makeTagsFrom(tags, enumeration);
	return tags;
    }

    public static ProcessEvent byTag(String s) {
	if (Utilities.getOperatingSystem() == Utilities.OS_LINUX)
	    return (ProcessEvent) byTagHelp(linux_enumeration, s);
	else
	    return (ProcessEvent) byTagHelp(enumeration, s);
    }

    public static ProcessEvent valueOf(String s) {
	if (Utilities.getOperatingSystem() == Utilities.OS_LINUX)
	    return (ProcessEvent) valueOfHelp(linux_enumeration, s);
	else
	    return (ProcessEvent) valueOfHelp(enumeration, s);
    }
}
