/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2008 Sun Microsystems, Inc.
 */
package org.netbeans.modules.dlight.spi.storage;

import org.netbeans.modules.dlight.spi.impl.DataStorageTypeAccessor;

/**
 * This object is marker object to define the following:
 * {@link  org.netbeans.modules.dlight.spi.collector.DataCollector} should
 * find the {@link  org.netbeans.modules.dlight.spi.storage.DataStorage}
 * it can put data into (it support),
 * {@link  org.netbeans.modules.dlight.spi.dataprovider.DataProvider} should
 * find the {@link  org.netbeans.modules.dlight.spi.storage.DataStorage} which
 * it can get data from and which contains data required.
 * Two objects of this typea are equals if they have the same  id returned by {@link  org.netbeans.modules.dlight.spi.storage.DataStorageType#getID() }
 *
 */
public final class DataStorageType {

  static{
    DataStorageTypeAccessor.setDefault(new DataStorageTypeAccessorImpl());
  }     

  private String id;

  DataStorageType(String id) {
    if (id == null) {
      throw new IllegalArgumentException("DataStorageType can be created only with NON NULL id"); // NOI18N
    }
    this.id = id;
  }

  /**
   * Unique type id
   * @return unique id
   */
  public String getID() {
    return id;
  }

  @Override
  public String toString() {
    return getID();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    DataStorageType other = (DataStorageType) obj;
    return other.getID().equals(this.id);
  }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 17 * hash + (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }

  private static final class DataStorageTypeAccessorImpl extends DataStorageTypeAccessor {

    @Override
    public DataStorageType createNew(String id) {
      return new DataStorageType(id);
    }
  }
}
