/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Sun Microsystems, Inc.
 */
package org.netbeans.modules.javacard.project.deps.ui;

import java.awt.Component;
import java.util.Map;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.openide.WizardDescriptor;
import org.openide.util.ChangeSupport;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.Parameters;
import org.openide.util.WeakListeners;

final class ChooseDependencyKindWizardPanel implements WizardDescriptor.Panel<Map<String,Object>>, ChangeListener {
    private final WizardDescriptor wiz;
    public ChooseDependencyKindWizardPanel(WizardDescriptor wiz) {
        this.wiz = wiz;
        Parameters.notNull ("wiz", wiz); //NOI18N
    }

    private ChooseDependencyKindPanelVisual component;

    public Component getComponent() {
        if (component == null) {
            component = new ChooseDependencyKindPanelVisual(wiz);
            component.addChangeListener (WeakListeners.change(this, component));
        }
        return component;
    }

    public HelpCtx getHelp() {
        return HelpCtx.DEFAULT_HELP;
    }

    public boolean isValid() {
        boolean result = component == null ? false : component.getDependencyKind() != null;
        if (!result) {
            wiz.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, NbBundle.getMessage(ChooseDependencyKindWizardPanel.class,
                    "ERR_CHOOSE_DEP_KIND")); //NOI18N
        } else {
            wiz.putProperty(WizardDescriptor.PROP_ERROR_MESSAGE, null);
        }
        return result;
    }

    private final ChangeSupport supp = new ChangeSupport(this);
    public final void addChangeListener(ChangeListener l) {
        supp.addChangeListener(l);
    }

    public final void removeChangeListener(ChangeListener l) {
        supp.removeChangeListener(l);
    }

    static final String PROP_DEP_KIND = "depKind"; //NOI18N
    public void readSettings(Map<String, Object> settings) {
        getComponent();
        InitialDepKind kind = (InitialDepKind) settings.get(PROP_DEP_KIND);
        component.setDependencyKind(kind);
    }

    public void storeSettings(Map<String, Object> settings) {
        if (component != null) {
            settings.put(PROP_DEP_KIND, component.getDependencyKind());
        }
    }

    public void stateChanged(ChangeEvent e) {
        supp.fireChange();
    }
}
