/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2011 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2011 Sun Microsystems, Inc.
 */

package org.netbeans.modules.maven;

import java.io.File;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.junit.NbTestCase;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.test.TestFileUtils;

public class NbMavenProjectFactoryTest extends NbTestCase {

    public NbMavenProjectFactoryTest(String n) {
        super(n);
    }

    protected @Override void setUp() throws Exception {
        clearWorkDir();
    }

    public void testIsProjectRegular() throws Exception {
        File d = getWorkDir();
        TestFileUtils.writeFile(new File(d, "pom.xml"), "...");
        assertTrue(ProjectManager.getDefault().isProject(FileUtil.toFileObject(d)));
    }

    public void testIsProjectNotDiskFile() throws Exception {
        FileObject r = FileUtil.createMemoryFileSystem().getRoot();
        r.createData("pom.xml");
        assertFalse(ProjectManager.getDefault().isProject(r));
    }

    public void testIsProjectArchetypeResources() throws Exception {
        File d1 = getWorkDir();
        TestFileUtils.writeFile(new File(d1, "pom.xml"), "...");
        assertTrue(ProjectManager.getDefault().isProject(FileUtil.toFileObject(d1)));
        File d2 = new File(d1, "src/main/resources/archetype-resources");
        TestFileUtils.writeFile(new File(d2, "pom.xml"), "...");
        assertFalse(ProjectManager.getDefault().isProject(FileUtil.toFileObject(d2)));
        File d3 = new File(d2, "subprj");
        TestFileUtils.writeFile(new File(d3, "pom.xml"), "...");
        assertFalse(ProjectManager.getDefault().isProject(FileUtil.toFileObject(d3)));
    }

    public void testIsProjectTargetDir() throws Exception { // #198162
        File d1 = getWorkDir();
        TestFileUtils.writeFile(new File(d1, "pom.xml"), "...");
        assertTrue(ProjectManager.getDefault().isProject(FileUtil.toFileObject(d1)));
        File d2 = new File(d1, "target");
        TestFileUtils.writeFile(new File(d2, "pom.xml"), "...");
        assertFalse(ProjectManager.getDefault().isProject(FileUtil.toFileObject(d2)));
    }

}
