/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2011 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2011 Sun Microsystems, Inc.
 */

package org.netbeans.modules.maven.spi.nodes;

import java.io.File;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import org.netbeans.junit.NbTestCase;
import org.netbeans.modules.maven.embedder.EmbedderFactory;
import org.openide.cookies.OpenCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.util.test.TestFileUtils;

public class NodeUtilsTest extends NbTestCase {

    public NodeUtilsTest(String n) {
        super(n);
    }

    public void testReadOnlyLocalRepositoryFile() throws Exception {
        File repo = new File(EmbedderFactory.getProjectEmbedder().getLocalRepository().getBasedir());
        File f = TestFileUtils.writeFile(new File(repo, "ant/ant/1.5.1/ant-1.5.1.jar.sha1"), "a50e3e050a6e78e0656edc183435b9773f53ce78");
        FileObject rw = FileUtil.toFileObject(f);
        FileObject ro = NodeUtils.readOnlyLocalRepositoryFile(rw);
        assertNotSame(rw, ro);
        assertFalse(ro.canWrite());
        assertNotNull(DataObject.find(ro).getLookup().lookup(OpenCookie.class));
        f = TestFileUtils.writeFile(new File(repo, "ant/ant/1.5.1/ant-1.5.1.pom.sha1"), "0ffdb41f140a621beeec4dc81b3d3ecaee085d28");
        rw = FileUtil.toFileObject(f);
        ro = NodeUtils.readOnlyLocalRepositoryFile(rw);
        assertNotSame(rw, ro);
        assertFalse(ro.canWrite());
        assertNotNull(DataObject.find(ro).getLookup().lookup(OpenCookie.class));
        assertSame(ro, NodeUtils.readOnlyLocalRepositoryFile(rw));
        FileObject skip = FileUtil.toFileObject(new File(repo, "ant/ant/1.5.1"));
        assertNotNull(skip);
        assertSame(skip, NodeUtils.readOnlyLocalRepositoryFile(skip));
        File stuff = TestFileUtils.writeFile(new File(getWorkDir(), "stuff"), "stuff");
        skip = FileUtil.toFileObject(stuff);
        assertNotNull(skip);
        assertSame(skip, NodeUtils.readOnlyLocalRepositoryFile(skip));
        Reference<?> r = new WeakReference<Object>(ro.getFileSystem());
        ro = null;
        assertGC("can collect FS", r);
    }

}
