/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2010 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package demo.rest.mvn.restsample.explorer;

import demo.rest.mvn.restsample.Message;
import demo.rest.mvn.restsample.client.MessageBoardClient;
import demo.rest.mvn.restsample.editor.MsgDetailsTopComponent;
import java.io.IOException;
import java.util.List;
import org.openide.actions.DeleteAction;
import org.openide.actions.OpenAction;
import org.openide.cookies.OpenCookie;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.actions.SystemAction;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author lukas
 */
public class MsgChildFactory extends ChildFactory<Message> {

    private List<Message> msgs;

    public MsgChildFactory(List<Message> msgs) {
        this.msgs = msgs;
    }

    @Override
    protected boolean createKeys(List<Message> list) {
        for (Message msg : msgs) {
            list.add(msg);
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(final Message key) {
        Node node = new AbstractNode(Children.LEAF, Lookups.fixed(
                key,
                new OpenCookie() {

                    @Override
                    public void open() {
                        MsgDetailsTopComponent win = MsgDetailsTopComponent.findInstance();
                        win.open();
                    }
                })) {

            @Override
            public SystemAction[] getActions() {
                SystemAction[] saa = new SystemAction[2];
                saa[0] = SystemAction.get(OpenAction.class);
                saa[1] = SystemAction.get(DeleteAction.class);
                return saa;
            }

            @Override
            public boolean canDestroy() {
                return true;
            }

            @Override
            public void destroy() throws IOException {
                MessageBoardClient client = new MessageBoardClient();
                client.deleteMessage(key);
                ExplorerTopComponent.refresh();
            }

        };
        node.setDisplayName(key.getMessage());
        node.setShortDescription(key.getCreated().toString());
        return node;
    }
}
