/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.websvc.rest.support;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.net.URL;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.SwingUtilities;
import org.apache.tools.ant.module.api.support.ActionUtils;
import org.netbeans.api.java.source.JavaSource;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.api.project.SourceGroup;
import org.netbeans.modules.web.api.webmodule.WebProjectConstants;
import org.netbeans.modules.websvc.api.support.LogUtils;
import org.netbeans.modules.websvc.rest.nodes.TestRestServicesAction;
import org.netbeans.modules.websvc.rest.spi.RestSupport;
import org.netbeans.spi.project.support.ant.GeneratedFilesHelper;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.awt.HtmlBrowser;
import org.openide.cookies.LineCookie;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.text.Line;
import org.openide.text.Line.ShowOpenType;
import org.openide.text.Line.ShowVisibilityType;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 *
 * @author Peter Liu
 */
public class Utils {
    
    public static String stripPackageName(String name) {
        int genericIndex = name.indexOf("<");//NOI18N
        int index = 0;
        if (genericIndex > 0) {
            String className = name.substring(0, genericIndex);
            index = className.lastIndexOf("."); //NOI18N
        } else {
            index = name.lastIndexOf("."); //NOI18N
        }
        if (index > 0) {
            return name.substring(index+1);
        }
        return name;
    }
    
    public static Collection<String> sortKeys(Collection<String> keys) {
        Collection<String> sortedKeys = new TreeSet<String>(
                new Comparator<String> () {
            public int compare(String str1, String str2) {
                return str1.compareTo(str2);
            }
        });
        
        sortedKeys.addAll(keys);
        return sortedKeys;
    }
    
    public static void showMethod(FileObject source, String methodName) {
        try {
            DataObject dataObj = DataObject.find(source);          
            JavaSource javaSource = JavaSource.forFileObject(source);
            
            // Force a save to make sure to make sure the line position in
            // the editor is in sync with the java source.
            SaveCookie sc = (SaveCookie) dataObj.getCookie(SaveCookie.class);
     
            if (sc != null) {
                sc.save();
            }
            
            LineCookie lc = (LineCookie) dataObj.getCookie(LineCookie.class);
            
            if (lc != null) {
                final long[] position = JavaSourceHelper.getPosition(javaSource, methodName);
                final Line line = lc.getLineSet().getOriginal((int) position[0]);
                
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        line.show(ShowOpenType.OPEN, ShowVisibilityType.NONE, (int) position[1]);
                    }
                });
            }
        } catch (Exception de) {
            Exceptions.printStackTrace(de);
        }    
    }

    public static Method getValueOfMethod(Class type) {
        try {
            Method method = type.getDeclaredMethod("valueOf", String.class);
            if (method == null || ! Modifier.isStatic(method.getModifiers())) {
                return null;
            }
            return method;
        } catch (Exception e) {
            return null;
        }
    }
    
    public static Constructor getConstructorWithStringParam(Class type) {
        try {
            return type.getConstructor(String.class);
        } catch (Exception e) {
            return null;
        }
    }
    
    /** Finds all projects in given lookup. If the command is not null it will check 
     * whther given command is enabled on all projects. If and only if all projects
     * have the command supported it will return array including the project. If there
     * is one project with the command disabled it will return empty array.
     */
    public static Project[] getProjectsFromLookup(Lookup lookup) {    
        Set<Project> result = new HashSet<Project>();
        for (Project p : lookup.lookupAll(Project.class)) {
            result.add(p);
        }
        // Now try to guess the project from dataobjects
        for (DataObject dObj : lookup.lookupAll(DataObject.class)) {
            FileObject fObj = dObj.getPrimaryFile();
            Project p = FileOwnerQuery.getOwner(fObj);
            if ( p != null ) {
                result.add( p );
            }
        }
        Project[] projectsArray = result.toArray(new Project[result.size()]);
        return projectsArray;
    }

    public static FileObject findBuildXml(Project project) {
        return project.getProjectDirectory().getFileObject(
                GeneratedFilesHelper.BUILD_XML_PATH);
    }
    
    public static void testRestWebService(Project project) {
        if ( project == null ){
            return;
        }
        
        TestRestTargetPanel panel = new TestRestTargetPanel(project);
        DialogDescriptor descriptor = new DialogDescriptor(panel,
                NbBundle.getMessage(Utils.class, 
                        "TTL_SelectTarget"));
        panel.setDescriptor( descriptor);
        if ( !DialogDisplayer.getDefault().notify(descriptor).equals(
                NotifyDescriptor.OK_OPTION)) 
        {
            return;
        }
        
        if (panel.isRemote()) {
            generateRemoteTester(project, panel.getProject());
        }
        else {
            generateLocalTester(project);
        }
    }
    
    private static void generateRemoteTester( Project restProject, Project remoteProject ) {
        RestSupport rs = remoteProject.getLookup().lookup(RestSupport.class);
        RestSupport localSupport = restProject.getLookup().lookup(RestSupport.class);
        SourceGroup[] sourceGroups = ProjectUtils.getSources(remoteProject).
            getSourceGroups(WebProjectConstants.TYPE_DOC_ROOT);
        SourceGroup sourceGroup = sourceGroups[0];
        FileObject rootFolder = sourceGroup.getRootFolder();
        try {
            FileObject testFO = rs.generateTestClient(FileUtil.toFile( rootFolder ),
                   localSupport.getBaseURL() );
            localSupport.deploy();
            rs.deploy();
            URL url = new URL( rs.getContextRootURL()+testFO.getNameExt());
            if (url != null) {
                HtmlBrowser.URLDisplayer.getDefault().showURL(
                        url);
            }
        }
        catch(IOException e ){
            Logger.getLogger( TestRestServicesAction.class.getName()).log( 
                    Level.WARNING, null, e);
        }
    }

    private static void generateLocalTester( Project prj ) {
        FileObject buildFo = findBuildXml(prj);
        if (buildFo != null) {
            try {
                Properties p = setupTestRestBeans(prj);
                ActionUtils.runTarget(buildFo,
                                new String[] { RestSupport.COMMAND_TEST_RESTBEANS },
                                p);
            }
            catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
        else {
            // if there is a rest support (e.g. in Maven projects)
            RestSupport rs = prj.getLookup().lookup(RestSupport.class);
            if (rs != null) {
                try {
                    FileObject testFO = rs.generateTestClient(
                            rs.getLocalTargetTestRest(), rs.getBaseURL());
                    rs.deploy();
                    if (testFO != null) {
                        URL url = testFO.getURL();
                        if (url != null) {
                            HtmlBrowser.URLDisplayer.getDefault().showURL(
                                    url);
                        }
                    }
                }
                catch (IOException ex) {
                    Exceptions.printStackTrace(ex);
                }
            }
        }

        // logging usage of action
        Object[] params = new Object[2];
        params[0] = LogUtils.WS_STACK_JAXRS;
        params[1] = "TEST REST"; // NOI18N
        LogUtils.logWsAction(params);
    }
    
    private static Properties setupTestRestBeans(Project project) throws IOException {
        Properties p = new Properties();
        p.setProperty(RestSupport.PROP_BASE_URL_TOKEN, RestSupport.BASE_URL_TOKEN);

        RestSupport rs = project.getLookup().lookup(RestSupport.class);
        if (rs != null) {
            try {
                String applicationPath = rs.getApplicationPath();
                if (applicationPath != null) {
                    if (!applicationPath.startsWith("/")) {
                        applicationPath = "/"+applicationPath;
                    }
                    p.setProperty(RestSupport.PROP_APPLICATION_PATH, applicationPath);
                }
                File testdir = rs.getLocalTargetTestRest();
                FileObject testFO = rs.generateTestClient(testdir);
                p.setProperty(RestSupport.PROP_RESTBEANS_TEST_URL, 
                        testFO.getURL().toString());
                p.setProperty(RestSupport.PROP_RESTBEANS_TEST_FILE, 
                        FileUtil.toFile(testFO).getAbsolutePath());
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            }
        }
        return p;
    }
    
}

